import numpy as np
import matplotlib
import matplotlib.pyplot as plt

from scipy.linalg import expm, solve
from numpy.linalg import norm

# Modelparameter
epsilon = 0.001
omega = 0.01
t_end = 10**5
n_steps = 300

def expEV(u0, du0dt, t_end, n_steps):
    """ Exponentielles Rosenbrock-Euler-Verfahren: O(h^2)

    Input:
        n_steps -- Anzahl Zeitschritte

        t_end      -- Endzeit
        u0      -- Anfangswert
        du0dt   -- Anfangswert von u'(0)

    Output:
        ts     -- [to ... te]
        y      -- Loesung y
    """
    ##########################
    ##### KODE HIER ##########
    return np.linspace(0,t_end, n_steps), np.zeros((n_steps,2)) # DIES LOESCHEN
    ##########################
def run_and_plot(u0, du0dt, t_end, n_steps, name):


    t, y = expEV(u0, du0dt, t_end, n_steps)


    plt.title("Exp Euler, t_end=%f, n_steps=%d" % (t_end, n_steps))
    plt.plot(t, y[:,0], label='Exp Euler, n_steps=%d' % n_steps)
    plt.xlabel("t")
    plt.ylabel("q")
    plt.legend()
    plt.savefig('exp_%s_t_q.png' % name)
    plt.show()
    plt.close("all")

    plt.figure()
    plt.plot(y[:,0], y[:,1], label='Exp Euler, n_steps=%d' % n_steps)
    plt.title("Exp Euler, t_end=%f, n_steps=%d" % (t_end, n_steps))
    plt.xlabel("q")
    plt.ylabel("p")
    plt.legend()
    plt.savefig('exp_%s_q_p.png' % name)
    plt.show()
    plt.close("all")


    E_tot = 0.5*y[:,1]**2 + (1 - np.cos(y[:,0]))
    plt.figure()
    plt.title("Exp Euler, t_end=%f, n_steps=%d" % (t_end, n_steps))
    plt.plot(t, E_tot, label='Exp Euler, n_steps=%d' % n_steps)
    plt.xlabel("t")
    plt.ylabel("E")
    plt.legend()
    plt.savefig('exp_%s_t_E.png' % name)
    plt.show()
    plt.close("all")


if __name__ == '__main__':
    # Initialwert
    u0 = 1
    du0dt = 0



    ## Aufgabe c), Teil 1)
    t_end = 10
    n_steps = 4000
    ## Berechnen und plotten Sie hier die Loesung
    run_and_plot(u0, du0dt, t_end, n_steps, 'stable')

    ## Aufgabe c), Teil 2)
    ##############
    t_end = 9000
    ##############
    n_steps = 4000
    ## Berechnen und plotten Sie hier die Loesung
    run_and_plot(u0, du0dt, t_end, n_steps, 'unstable')
