# coding: utf-8

import numpy as np
import matplotlib.pyplot as plt

from simple_splitting import integrate
from splitting_parameters import splitting_parameters

# Modelparameter
epsilon = 0.001
omega = 0.01

def splitting_step(Phi_a, Phi_b, y0, dt, a, b):
    """Allgemeines Splittingverfahren, mit Parameter `a`, `b`.

    Input:
        Phi_a: Loesungsoperator A, Signatur `(y0, dt)`.
        Phi_a: Loesungsoperator B, Signatur `(y0, dt)`.

           y0: Aktuelle Approximation der Loesung.
           dt: wie ueblich.

            a: Relative Schrittweite fuer `Phi_a`.
            b: Relative Schrittweite fuer `Phi_b`.

    Output:
        y1: Approximative Loesung der ODE zur Zeit `t+dt`.
    """

    # Tip: defensiv programmieren:
    if a.size != b.size:
        raise Exception("Dimensions mismatch![%s, %s]".format(str(a), str(b)))

    y = y0
    for s in range(a.size):
        y = Phi_a(y, a[s]*dt)
        y = Phi_b(y, b[s]*dt)

    return y

def splitting_method(Phi_a, Phi_b, y0, t_end, n_steps, key):
    """Allgemeines Splittingverfahren, mit Parameter `a`, `b`.

    Integriert bis t_end mit n_steps.

    Input:
        Phi_a: Loesungsoperator A, Signatur `(y0, dt)`.
        Phi_a: Loesungsoperator B, Signatur `(y0, dt)`.

           y0: Aktuelle Approximation der Loesung.
           dt: wie ueblich.

            a: Relative Schrittweite fuer `Phi_a`.
            b: Relative Schrittweite fuer `Phi_b`.

    Output:
        y1: Approximative Loesung der ODE zur Zeit `t+dt`.
    """

    a, b = splitting_parameters(key)
    method = lambda y, dt: splitting_step(Phi_a, Phi_b, y, dt, a, b)

    return integrate(method, y0, t_end, n_steps)


def strang_splitting(u0, du0dt, t_end, n_steps):
    ######### AUFGABE b)########
    ######### KODE HIER ########
    return np.linspace(0,t_end, n_steps), np.zeros((n_steps,2)) # DIES LOESCHEN
    ############################






def run_and_plot(u0, du0dt, t_end, n_steps, name):
    t, y = strang_splitting(u0, du0dt, t_end, n_steps)


    plt.title("SS, t_end=%f, n_steps=%d" % (t_end, n_steps))
    plt.plot(t, y[:,0], label='SS, n_steps=%d' % n_steps)
    plt.xlabel("t")
    plt.ylabel("q")
    plt.legend()
    plt.savefig('ss_%s_t_q.png' % name)
    plt.show()
    plt.close("all")

    plt.figure()
    plt.plot(y[:,0], y[:,1], label='SS, n_steps=%d' % n_steps)
    plt.title("SS, t_end=%f, n_steps=%d" % (t_end, n_steps))
    plt.xlabel("q")
    plt.ylabel("p")
    plt.legend()
    plt.savefig('ss_%s_q_p.png' % name)
    plt.show()
    plt.close("all")


    E_tot = 0.5*y[:,1]**2 + (1 - np.cos(y[:,0]))
    plt.figure()
    plt.title("SS, t_end=%f, n_steps=%d" % (t_end, n_steps))
    plt.plot(t, E_tot, label='SS, n_steps=%d' % n_steps)
    plt.xlabel("t")
    plt.ylabel("E")
    plt.legend()
    plt.savefig('ss_%s_t_E.png' % name)
    plt.show()
    plt.close("all")


if __name__ == '__main__':
    # Initialwert
    u0 = 1
    du0dt = 0



    ## Aufgabe c), Teil 1)
    t_end = 10
    n_steps = 4000
    ## Berechnen und plotten Sie hier die Loesung
    run_and_plot(u0, du0dt, t_end, n_steps, 'stable')

    ## Aufgabe c), Teil 2)
    ##############
    t_end = 10 ### Finden Sie hier einen T, bei dem die Methode mit n_steps=4000 offensichtlich eine falsche Loesung produziert.
    ##############
    n_steps = 4000
    ## Berechnen und plotten Sie hier die Loesung
    run_and_plot(u0, du0dt, t_end, n_steps, 'unstable')
